# frozen_string_literal: true

require 'apple_core/refinements/string'

module  AppleCore
module  ActionController
module  ResourceNaming
  using ::AppleCore::Refinements::String

  CONTROLLER_RESOURCE_NAME_PATTERN = /
    \A                                        # Beginning of String
      (?:
        (?<modules>                           #   Full Modules
          (?:                                 #     Match Root Module (Optional)
            (?<root_module>
              [^\:]+                          #         The Module Name
              (?=\:\:)                        #         Which Must Be Followed By '::'
            )
            (?:\:\:)?                         #       Up Through the '::' But Don't
                                              #       Capture If It's The Sole Module
          )?
          (?<submodules>                      #   Match Submodule List (Optional)
            (                                 #     There May Be Zero or More of These
              [^\:]+                          #       The Module Name
              \:\:                            #       Up Through the '::'
            )*?
            (?<last_submodule>                #     The Final Submodule (Optional)
              [^\:]+                          #       The Module Name
              (?=\:\:)                        #       Which Must Be Followed By '::'
            )?
          )?
        )
        \:\:                                  #   Required Final Separator
      )?
      (?<controller_name>
        (?<resource_name>\w+?)                #     Base Resource Name
        (?:Index|Indicies)?                   #     Optional Index Suffix
      )
      Controller                              #   Literal 'Controller'
    \z                                        # End of String
  /x

  def self.included(base)
    base.extend(self)
  end

  def singular_underscored_base_resource_name
    @singular_underscored_base_resource_name ||= singular_resource_base_class_name
                                                   .underscore
                                                   .downcase
  end

  def plural_underscored_base_resource_name
    @plural_underscored_base_resource_name ||= plural_resource_base_class_name
                                                 .underscore
                                                 .downcase
  end

  def singular_resource_base_class_name
    @singular_resource_base_class_name ||= plural_resource_base_class_name
                                             .singularize
  end

  def plural_resource_base_class_name
    @plural_resource_base_class_name ||= resource_name
                                           .pluralize
  end

  def singular_resource_class_name
    @singular_resource_class_name ||= [
                                        name_components['root_module'],
                                        resource_name,
                                      ]
                                        .compact
                                        .join('::')
  end

  def plural_resource_class_name
    @plural_resource_class_name ||= singular_resource_class_name
                                      .pluralize
  end

  def name_components
    @name_components ||= name.match(CONTROLLER_RESOURCE_NAME_PATTERN).named_captures
  end

  def resource_name
    @resource_name ||= name_components['resource_name']
                         .singularize
  end
end
end
end
